<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://wordpress.org/plugins/cb-vegas/
 * @since      0.1.0
 * @package    CB_Vegas
 * @subpackage CB_Vegas/admin
 */

/**
 * The admin-specific functionality of the plugin.
 * Initiates all admin-related modules such as
 * help tab,
 * wp support,
 * meta box,
 * settings page
 * and the settings.
 *
 * @package    CB_Vegas
 * @subpackage CB_Vegas/admin
 * @author     demispatti
 */
class CB_Vegas_Admin {

	/**
	 * The name of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_name
	 */
	protected $plugin_name;

	/**
	 * The domain of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_domain
	 */
	protected $plugin_domain;

	/**
	 * The version of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_name
	 */
	protected $plugin_version;

	/**
	 * The reference to the loader object.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    object $loader
	 */
	protected $loader;

	/**
	 * The reference to the settings page object.
	 *
	 * @since  0.1.0
	 * @access public
	 * @var    object $settings_page
	 */
	public $settings_page;

	/**
	 * The name of the plugin.
	 *
	 * @since  0.1.0
	 * @access public
	 * @var    object $settings
	 */
	public $settings;

	/**
	 * Initializes the admin part.
	 *
	 * @since 0.1.0
	 * @param string $plugin_name
	 * @param string $plugin_domain
	 * @param string $plugin_version
	 * @param object $loader
	 */
	public function __construct($plugin_name, $plugin_domain, $plugin_version, $loader) {

		$this->plugin_name = $plugin_name;
		$this->plugin_domain = $plugin_domain;
		$this->plugin_version = $plugin_version;
		$this->loader = $loader;

		$this->load_dependencies();
		$this->define_help_tab();
		$this->define_wp_support();
		$this->define_meta_box();
		$this->define_settings_page();
		$this->define_settings();
	}

	/**
	 * Loads its dependencies.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function load_dependencies() {

		require_once plugin_dir_path(dirname(__FILE__)) . "admin/includes/class-cb-vegas-help-tab.php";

		require_once plugin_dir_path(dirname(__FILE__)) . "admin/includes/class-cb-vegas-wp-support.php";

		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/includes/class-cb-vegas-meta-box.php';

		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/menu/class-cb-vegas-settings-page.php';
	}

	/**
	 * Registers the style with WordPress.
	 *
	 * @hooked_action
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function enqueue_styles() {

		wp_enqueue_style(
			$this->plugin_name . '-style-css',
			plugin_dir_url(__FILE__) . 'css/style.css',
			array(),
			$this->plugin_version,
			'all'
		);
	}

	/**
	 * Registers the scripts with WordPress.
	 *
	 * @hooked_action
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function enqueue_scripts() {

		if ( ! wp_script_is('fancySelect.js')) {
			// Fancy Select
			wp_enqueue_script(
				$this->plugin_name . '-inc-fancy-select-js',
				plugin_dir_url(__FILE__) . '../vendor/fancy-select/fancySelect.js',
				array('jquery'),
				$this->plugin_version,
				false
			);
		}

		wp_enqueue_script(
			$this->plugin_name . '-meta-box-js',
			plugin_dir_url(__FILE__) . 'js/meta-box.js',
			array('jquery', $this->plugin_name . '-inc-fancy-select-js'),
			$this->plugin_version,
			false
		);
	}

	/**
	 * Creates a reference to the "help tab class" and hooks the initial function with WordPress.
	 *
	 * @see    admin/includes/class-cb-vegas-help-tab.php
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function define_help_tab() {

		$Help_Tab = new CB_Vegas_Help_Tab($this->get_plugin_domain());

		$this->loader->add_action('in_admin_header', $Help_Tab, 'add_tab');
	}

	/**
	 * Registers the action to execute on the object regarding the post type support.
	 *
	 * @since    0.1.0
	 * @access   private
	 * @return   void
	 */
	private function define_wp_support() {

		$post_type_support = new CB_Vegas_WP_Support();

		$this->loader->add_action('after_setup_theme', $post_type_support, 'enable_post_type_support', 1);
		$this->loader->add_action('after_setup_theme', $post_type_support, 'add_theme_support', 2);
	}

	/**
	 * Registers the meta box related features to execute them on the just created object.
	 *
	 * @since    0.1.0
	 * @access   private
	 * @return   void
	 */
	private function define_meta_box() {

		$Meta_Box = new CB_Vegas_Meta_Box($this->get_plugin_domain());

		$this->loader->add_action('load-post.php', $Meta_Box, 'load_post');
		$this->loader->add_action('load-post-new.php', $Meta_Box, 'load_post');
	}

	/**
	 * Registers all necessary hooks and instanciates all necessary objects the settings page is made of.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function define_settings_page() {

		$this->settings_page = new CB_Vegas_Settings_Page($this->get_plugin_name(), $this->get_plugin_domain(), $this->get_plugin_domain(), $this->get_loader());

		$this->loader->add_action('admin_enqueue_scripts', $this->settings_page, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $this->settings_page, 'enqueue_scripts');
		$this->loader->add_action('admin_enqueue_scripts', $this->settings_page, 'define_localisations', 100);
		$this->loader->add_action('admin_menu', $this->settings_page, 'add_settings_page');
		$this->loader->add_action('admin_notices', $this->settings_page, 'admin_notice_display');

		$this->loader->add_action('wp_ajax_add_slide', $this->settings_page, 'add_slide');
		$this->loader->add_action('wp_ajax_duplicate_slide', $this->settings_page, 'duplicate_slide');
		$this->loader->add_action('wp_ajax_sort_slides', $this->settings_page, 'sort_slides');
		$this->loader->add_action('wp_ajax_remove_slide', $this->settings_page, 'remove_slide');
		$this->loader->add_action('wp_ajax_add_slideshow', $this->settings_page, 'add_slideshow');
		$this->loader->add_action('wp_ajax_duplicate_slideshow', $this->settings_page, 'duplicate_slideshow');
		$this->loader->add_action('wp_ajax_sort_slideshows', $this->settings_page, 'sort_slideshows');
		$this->loader->add_action('wp_ajax_remove_slideshow', $this->settings_page, 'remove_slideshow');
	}

	/**
	 * Defines the hooks that initiate the settings api.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function define_settings() {

		$this->settings = new CB_Vegas_Settings($this->get_plugin_domain(), $this->get_settings_page());

		$this->loader->add_action('admin_init', $this->settings, 'register_settings', 10);
		$this->loader->add_action('admin_init', $this->settings, 'initialize_settings', 20);
	}

	/**
	 * Returns the name of the plugin.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return string $plugin_name
	 */
	public function get_plugin_name() {

		return $this->plugin_name;
	}

	/**
	 * Returns the domain of the plugin.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return string $plugin_domain
	 */
	public function get_plugin_domain() {

		return $this->plugin_domain;
	}

	/**
	 * Returns the version of the plugin.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return string $plugin_version
	 */
	public function get_plugin_version() {

		return $this->plugin_version;
	}

	/**
	 * Returns the reference to the class responsible for registering the hooks with WordPress.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return object $loader
	 */
	public function get_loader() {

		return $this->loader;
	}

	/**
	 * Returns the reference to the class responsible for the settings page.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return object $settings_page
	 */
	public function get_settings_page() {

		return $this->settings_page;
	}

	/**
	 * Adds support, rating, and donation links to the plugin row meta on the plugins admin screen.
	 *
	 * @hooked_action
	 *
	 * @since    0.1.0
	 * @access   public
	 * @return   array
	 *
	 * @param    array  $meta
	 * @param    string $file
	 */
	public function plugin_row_meta($meta, $file) {

		$plugin = plugin_basename('cb-vegas/cb-vegas.php');

		if ($file == $plugin) {
			$meta[] = '<a href="https://wordpress.org/support/plugin/cb-vegas" target="_blank">' . __('Plugin Support', $this->plugin_domain) . '</a>';
			$meta[] = '<a href="https://wordpress.org/plugins/cb-vegas/" target="_blank">' . __('Rate Plugin', $this->plugin_domain) . '</a>';
			$meta[] = '<a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=XLMMS7C62S76Q" target="_blank">' . __('Donate', $this->plugin_domain) . '</a>';
		}

		return $meta;
	}
}
