<?php

/**
 * The class responsible for creating and displaying the help tab on this plugin's menu page.
 *
 * @link              https://wordpress.org/plugins/cb-vegas/
 * @since             0.1.0
 * @package           CB_Vegas
 * @subpackage        CB_Vegas/admin/includes
 * Author:            demispatti <demis@demispatti.ch>
 * Author URI:        http://demispatti.ch
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */
class CB_Vegas_Help_Tab {

	/**
	 * The name of the domain.
	 *
	 * @since  0.1.0
	 * @access private
	 * @var    string $plugin_domain
	 */
	private $plugin_domain;

	/**
	 * The array containing the title and the content of the help tab.
	 *
	 * @since  0.1.0
	 * @access private
	 * @var    array $tabs
	 */
	private $tabs;

	/**
	 * Kicks off the help tab.
	 *
	 * @since  0.1.0
	 * @access public
	 * @param string $plugin_domain
	 * @return void
	 */
	public function __construct($plugin_domain) {

		$this->plugin_domain = $plugin_domain;

		$this->set_tab();
		$this->initialize();
	}

	/**
	 * Sets the title of the help tab.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function set_tab() {

		$this->tabs = array(__('HELP', $this->plugin_domain) => array('title' => __('cbVegas help', $this->plugin_domain)));
	}

	/**
	 * Initializes the help tab.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function initialize() {

		// We do only add the help tab on the plugin options page.
		if (isset($_REQUEST['page']) && $_REQUEST['page'] == 'cb_vegas_settings_page') {

			add_action("load-{$GLOBALS['pagenow']}", array($this, 'add_tab'));
		}
	}

	/**
	 * Adds the tab to the current page.
	 *
	 * @hooked_action
	 *
	 * @uses WordPress Administration Screen API screen.php / add_help_tab()
	 * @uses display_content_callback()
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function add_tab() {

		foreach ($this->tabs as $id => $data) {

			$title = __($data['title'], $this->plugin_domain);

			get_current_screen()->add_help_tab(array(
				'id'      => $id,
				'title'   => __($title, $this->plugin_domain),
				'content' => $this->display_content_callback(),
			));
		}
	}

	/**
	 * Returns the content of the help tab.
	 *
	 * @used-by add_help_tab()
	 * @since  0.1.0
	 * @access private
	 * @return string $html
	 */
	private function display_content_callback() {

		$html = '<p>' . __("Thanks for using this plugin.", $this->plugin_domain) . '</p>';

		$html .= '<p>' . __("Please note: This software makes constant use of ajax functionalities. When you add, remove, duplicate or rearrange the slides and slideshows, the result will be shown instantly. Your input is being stored in a temporary database entry. This means that you have to save the changes manually (with the \"Save Changes\" button) to finally save your settings. On the other hand, you can throw your \"temporary\" changes by refreshing the page.", $this->plugin_domain) . '</p>';

		$html .= '<p>' . __("If you set a slideshow global, it will be displayed on every regular page and post and override any slideshow including those that were defined on a per post / per page basis.", $this->plugin_domain) . '</p>';

		$html .= '<p>' . __("If you set a slideshow as fallback, the behaviour is the same as if you set it as global slideshow, except that slideshows that are defined on a per page / post basis are not overridden at all. On all other regular pages and posts the slideshow you defined as fallback will be displayed.", $this->plugin_domain) . '</p>';

		$html .= '<p>' . __("All other settings are part of the 'Vegas Background Slideshow'. For help regarding 'Vegas Background Slideshow' itself, please refer to it's <a href='http://vegas.jaysalvat.com/documentation/setup/' target='_blank'>official documentation</a>.", $this->plugin_domain) . '</p>';

		$html .= '<p>' . __("If you have any questions, comments or issues regarding this plugin, please visit the <a href='https://wordpress.org/plugins/cb-vegas/' target='_blank'>plugin homepage</a>.", $this->plugin_domain) . '</p>';

		return $html;
	}

}
