<?php

/**
 * The class that defines the settings page.
 *
 * @link              https://wordpress.org/plugins/cb-vegas/
 * @since             0.1.0
 * @package           CB_Vegas
 * @subpackage        CB_Vegas/admin/menu
 * Author:            demispatti
 * Author URI:        http://demispatti.ch
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */
class CB_Vegas_Settings_Page {
	
	/**
	 * The name of the plugin.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var string $plugin_name
	 */
	protected $plugin_name;
	
	/**
	 * The domain of the plugin.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var string $plugin_domain
	 */
	protected $plugin_domain;
	
	/**
	 * The version of the plugin.
	 * @since    0.1.0
	 * @access   protected
	 * @var string $plugin_version
	 */
	protected $plugin_version;

	/**
	 * The reference to the loader class.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var      object $loader
	 */
	protected $loader;

	/**
	 * The reference to the settings class.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var object $settings
	 */
	public $settings;

	/**
	 * The reference to the options class.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var object $vegas_options
	 */
	public $vegas_options;

	/**
	 * The reference to the class that generates all the settings.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var object $settings_factory
	 */
	public $settings_factory;

	/**
	 * The reference to the class that's responsible for all ajax actions.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var object $ajax
	 */
	public $ajax;

	/**
	 * Kicks off the settings page.
	 *
	 * @since 0.1.0
	 * @param string $plugin_name
	 * @param string $plugin_domain
	 * @param string $plugin_version
	 * @param object $loader
	 */
	public function __construct($plugin_name, $plugin_domain, $plugin_version, $loader) {

		$this->plugin_name = $plugin_name;
		$this->plugin_domain = $plugin_domain;
		$this->plugin_version = $plugin_version;
		$this->loader = $loader;

		$this->load_dependencies();
		$this->add_ajax_actions();
		$this->init();
	}

	/**
	 * Loads it's dependencies.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function load_dependencies() {

		require_once plugin_dir_path(dirname(__FILE__)) . 'menu/includes/class-cb-vegas-settings.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'menu/includes/class-cb-vegas-localisations.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'menu/includes/settings/class-cb-vegas-ajax.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'menu/includes/settings/class-cb-vegas-options.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'menu/includes/settings/class-cb-vegas-settings-factory.php';

		$this->settings = new CB_Vegas_Settings($this->get_plugin_domain(), $this->get_settings_page());
		$this->vegas_options = new CB_Vegas_Options($this->get_plugin_domain(), $this->get_settings());
		$this->settings_factory = new CB_Vegas_Settings_Factory($this->get_plugin_domain(), $this->get_vegas_options(), $this->get_settings());
		$this->ajax = new CB_Vegas_Ajax($this->get_plugin_domain(), $this->get_vegas_options(), $this->get_settings(), $this->get_settings_factory());
	}

	/**
	 * Hooks all ajax functions with WordPress.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function add_ajax_actions() {

		add_action('wp_ajax_add_slide', array(&$this, 'add_slide'));
		add_action('wp_ajax_dupliacte_slide', array(&$this, 'duplicate_slide'));
		add_action('wp_ajax_sort_slides', array(&$this, 'sort_slides'));
		add_action('wp_ajax_remove_slide', array(&$this, 'remove_slide'));

		add_action('wp_ajax_add_slideshow', array(&$this, 'add_slideshow'));
		add_action('wp_ajax_duplicate_slideshow', array(&$this, 'duplicate_slideshow'));
		add_action('wp_ajax_sort_slideshows', array(&$this, 'sort_slideshows'));
		add_action('wp_ajax_remove_slideshow', array(&$this, 'remove_slideshow'));
	}

	/**
	 * Adds the settings page and initiates its localisation.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function init() {

		add_action('admin_menu', array(&$this, 'add_settings_page'));
		add_action('admin_enqueue_scripts', array(&$this, 'define_localisations'), 100);
		add_action('admin_notices', array(&$this, 'admin_notice_display'));
	}

	/**
	 * Registers the stylesheet for the settings page.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function enqueue_styles($hook_suffix) {

		// Menu
		if (isset($hook_suffix) && $hook_suffix === 'settings_page_cb_vegas_settings_page') {

			// Thickbox.
			wp_enqueue_style('thickbox');

			// Color Picker.
			wp_enqueue_style('wp-color-picker');

			// Don't load if already enqueued from somewhere else.
			if ( ! wp_style_is('alertify.core.css', 'enqueued')) {

				// Alertify
				wp_enqueue_style(
					$this->plugin_name . '-inc-alertify-core-css',
					plugin_dir_url(__FILE__) . '../../vendor/alertify/alertify.core.css',
					array(),
					$this->plugin_version,
					false
				);
			}

			if ( ! wp_style_is('font-awesome.css', 'enqueued')) {

				wp_enqueue_style(
					$this->plugin_name . '-inc-font-awesome',
					'https://netdna.bootstrapcdn.com/font-awesome/4.0.3/css/font-awesome.css',
					array(),
					$this->plugin_version,
					false
				);
			}
		}
	}

	/**
	 * Register the scripts for the settings menu.
	 *
	 * @since    0.1.0
	 * @param string $hook_suffix
	 * @return void
	 */
	public function enqueue_scripts($hook_suffix) {

		// We load this stuff only on the settings page.
		if (isset($hook_suffix) && $hook_suffix === 'settings_page_cb_vegas_settings_page') {

			// jQuery UI Components.
			$jquery_ui_libraries = array(
				'jquery-ui-core',
				'jquery-ui-widget',
				'jquery-ui-mouse',
				'jquery-ui-position',
				'jquery-ui-draggable',
				'jquery-ui-droppable',
				'jquery-ui-sortable',
				'jquery-ui-accordion',
				'jquery-ui-tooltip',
				'jquery-effects-core',
				'jquery-effects-bounce',
			);
			/*foreach ($jquery_ui_libraries as $jquery_ui_library) {
				wp_enqueue_script($jquery_ui_library);
			}*/

			// Media.
			wp_enqueue_media();

			// Color Picker.
			wp_enqueue_script('wp-color-picker');

			// Alertify.
			wp_enqueue_script(
				$this->plugin_name . '-inc-alertify-js',
				plugin_dir_url(__FILE__) . '../../vendor/alertify/alertify.js',
				array('jquery',),
				$this->plugin_version,
				false
			);

			// Ajax.
			$dependencies = array_merge(
				array('jquery', 'wp-color-picker', $this->plugin_name . '-inc-fancy-select-js', $this->plugin_name . '-inc-alertify-js'),
				$jquery_ui_libraries
			);
			wp_enqueue_script(
				$this->plugin_name . '-ajax-js',
				plugin_dir_url(__FILE__) . 'js/ajax.js',
				$dependencies
				/*array('jquery', 'wp-color-picker', $this->plugin_name . '-inc-fancy-select-js', $this->plugin_name . '-inc-alertify-js')*/,
				$this->plugin_version,
				false
			);

			// Main.
			$dependencies = array_merge(
				array('jquery', 'wp-color-picker', $this->plugin_name . '-inc-fancy-select-js'),
				$jquery_ui_libraries
			);
			wp_enqueue_script(
				$this->plugin_name . '-main-js',
				plugin_dir_url(__FILE__) . 'js/main.js',
				$dependencies
				/*array('jquery', 'wp-color-picker', $this->plugin_name . '-inc-fancy-select-js')*/,
				$this->plugin_version,
				false
			);
		}
	}

	/**
	 * Registers the settings page with WordPress.
	 *
	 * @hooked_action
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function add_settings_page() {

		add_options_page('cbVegas', 'cbVegas', 'manage_options', 'cb_vegas_settings_page', array(&$this, 'display'));
	}

	/**
	 * Runs the instance of the class that's responsible for localizing the settings page texts served via javascript.
	 *
	 * @hooked_action
	 * @since  0.1.0
	 * @access public
	 * @param  string $hook_suffix
	 * @return void
	 */
	public function define_localisations($hook_suffix) {

		if (isset($hook_suffix) && $hook_suffix === 'settings_page_cb_vegas_settings_page') {

			$Localisations = new CB_Vegas_Localisations($this->get_plugin_name(), $this->get_plugin_domain());

			$Localisations->run();
		}
	}

	/**
	 * Displays the validation errors in the admin notice area.
	 *
	 * @hooked_action
	 *
	 * @since  0.1.0
	 * @access public
	 * @uses   WP_Error/get_error_message()
	 * @return echo
	 */
	public function admin_notice_display() {

		// If there are any error-related transients
		if (false !== get_transient('cb_vegas_validation_transient')) {

			// Retrieves the error-array and the corresponding meta data
			$errors = get_transient('cb_vegas_validation_transient');

			foreach ($errors as $slide => $error) {

				foreach ($error as $key => $args) {
					// Extracts the error message and echoes it inside the admin notice area.
					if (is_wp_error($error[ $key ]['message'])) {

						$error_message = $error[ $key ]['message']->get_error_message();
						$slide_number = $slide + 1;// Converts the index number to slide number.

						// The actual error message.
						$html = '<div class="error notice is-dismissible"' . $error[ $key ]['notice_level'] . '">';
						$html .= '<p class="cb-vegas-validation-error" >' . $error[ $key ]['name'] . ', Slide Nr. ' . $slide_number . '</p>';
						$html .= '<p>' . $error_message . '</p>';
						$html .= '</div>';

						echo $html;
					}
				}
			}
			// Clean up
			delete_transient('cb_vegas_validation_transient');
		}
	}

	/**
	 * Renders the page for the menu.
	 *
	 * @callback
	 *
	 * @since  0.1.0
	 * @access public
	 * @return mixed
	 */
	public function display() {

		$slideshow_index = $this->get_slideshow_index();
		?>

		<div class="wrap">

			<div id="icon-themes" class="icon32"></div>

			<h3><?php echo __('Slideshows:', $this->plugin_domain); ?></h3>

			<div class="cb-vegas-menu-container">
				<?php $cb_vegas_sort_tabs_nonce = wp_create_nonce('cb_vegas_sort_tabs_nonce'); ?>
				<div class="cb-vegas-navigation-tabs-container" data-nonce="<?php echo $cb_vegas_sort_tabs_nonce ?>">
					<?php
					$this->settings_factory->create_navigation($slideshow_index);
					?>
				</div>
				<?php
				$this->settings_factory->create_add_slideshow_button();
				?>
			</div>

			<form id="cb-vegas-settings-form" method="POST" action="options.php" data-slideshow-index="<?php echo $slideshow_index ?>">
				<div id="controls">
					<?php
					$this->settings_factory->create_add_slide_button();
					//$this->settings_factory->create_duplicate_slideshow_button();
					$this->settings_factory->create_remove_slideshow_button();
					$this->settings_factory->create_submit_button();
					?>
				</div>
				<h3 style="display: block;"><?php echo __('Settings:', $this->plugin_domain); ?></h3>

				<div id="settings">
					<?php
					$this->settings_factory->create_slideshow_index_hidden_field($slideshow_index);
					$this->settings_factory->create_slideshow_uniqid_hidden_field($slideshow_index);
					$this->settings_factory->create_slideshow_name_setting($slideshow_index);
					?>
					<div class="cb-vegas-settings-slider">
						<?php
						$this->settings_factory->create_slideshow_is_global_setting($slideshow_index);
						$this->settings_factory->create_slideshow_is_fallback_setting($slideshow_index);
						$this->settings_factory->create_slideshow_is_autoplay_setting($slideshow_index);
						$this->settings_factory->create_slideshow_is_overlay_setting($slideshow_index);
						$this->settings_factory->create_slideshow_is_shuffle_setting($slideshow_index);
						$this->settings_factory->create_slideshow_is_timer_setting($slideshow_index);
						?>
					</div>
				</div>
				<?php $cb_vegas_sort_slides_nonce = wp_create_nonce('cb_vegas_sort_slides_nonce'); ?>
				<div id="cb-vegas-slides-container" data-nonce="<?php echo $cb_vegas_sort_slides_nonce ?>">
					<?php
					settings_fields('cb_vegas_options');
					do_settings_sections('cb_vegas_settings_section');
					do_settings_fields('cb_vegas_settings_page', 'cb_vegas_settings_section');
					?>
				</div>
			</form>

		</div>

		<?php

		$this->delete_current_slideshow_index_transient();
		$this->set_current_options_transient();
	}

	/**
	 * Deletes the current slideshow index transient.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function delete_current_slideshow_index_transient() {

		delete_transient('cb_vegas_current_slideshow_index_transient');
	}

	/**
	 * Sets a transient containing the current options.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function set_current_options_transient() {

		$options = get_option('cb_vegas_options');

		delete_transient('cb_vegas_current_options_transient');
		set_transient('cb_vegas_current_options_transient', $options, 1800);
	}

	/**
	 * Returns the slideshow index.
	 *
	 * @callback
	 *
	 * @since  0.1.0
	 * @access public
	 * @return mixed
	 */
	public function get_slideshow_index() {

		$current_slideshow_index = get_transient('cb_vegas_current_slideshow_index_transient');
		$slideshow_index = NULL;

		$allowed_actions = array('add_slideshow', 'add_slide', 'sort_slideshows', 'sort_slides', 'remove_slideshow', 'remove_slide');
		$request_action = isset($_REQUEST['action']) ? $_REQUEST['action'] : false;

		// If the user reordered the slideshows using ajax, then this is the current slideshow index.
		if (isset($_REQUEST['settings-updated']) && $_REQUEST['settings-updated'] === 'true') {

			$slideshow_index = get_transient('cb_vegas_current_slideshow_index_transient');
		} elseif (isset($_REQUEST['current_slideshow_index']) && $_REQUEST['current_slideshow_index'] !== '' && $_REQUEST['current_slideshow_index'] !== false) {

			$slideshow_index = $_REQUEST['current_slideshow_index'];
			// If this is true, we use this slideshow index.
		} else if (isset($_REQUEST['cb_vegas_options']['slideshow_index']) && $_REQUEST['cb_vegas_options']['slideshow_index'] !== '' && $_REQUEST['cb_vegas_options']['slideshow_index'] !== false) {

			$slideshow_index = $_REQUEST['cb_vegas_options']['slideshow_index'];
			// If this is true, we use this slideshow index.
		} else if (isset($_REQUEST['action']) && in_array($request_action, $allowed_actions)) {

			$slideshow_index = get_transient('cb_vegas_current_slideshow_index_transient');
			//$slideshow_index = $_REQUEST['cb_vegas_options']['slideshow_index'];
			set_transient('cb_vegas_current_slideshow_index_transient', $slideshow_index, 1800);
			// If the user calls a regular settings page tab, use that.
		} else if (isset($_REQUEST['tab']) && $_REQUEST['tab'] !== 'undefined') {

			$slideshow_index = $_REQUEST['tab'];
			// Else if there is a current value for the slideshow index, use this one and touch wood.
		} else if (false !== $current_slideshow_index && '' !== $current_slideshow_index && NULL !== $current_slideshow_index) {

			$slideshow_index = $current_slideshow_index;
			// Else we can assume a regular call to the plugin's settings page and we assign an index number of zero. Don't forget to touch wood.
		} else {

			$slideshow_index = 0;
		}
		// Necessary after settings got updated.
		set_transient('cb_vegas_current_slideshow_index_transient', $slideshow_index, 1800);

		return (int)$slideshow_index;
	}

	/**
	 * Returns the reference to the settings class.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return objct $settings
	 */
	public function get_settings() {

		return $this->settings;
	}

	/**
	 * Returns the reference to the settings page class.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return objct $settings_page
	 */
	public function get_settings_page() {

		return $this;
	}

	/**
	 * Returns the reference to the settings factory class.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return objct $settings_factory
	 */
	public function get_settings_factory() {

		return $this->settings_factory;
	}

	/**
	 * Returns the reference to the options class.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return objct $vegas_options
	 */
	public function get_vegas_options() {
		
		return $this->vegas_options;
	}

	/**
	 * Returns the plugin name.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return string $plugin_name
	 */
	public function get_plugin_name() {

		return $this->plugin_name;
	}

	/**
	 * Returns the domain of the plugin.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return string $plugin_domain
	 */
	public function get_plugin_domain() {

		return $this->plugin_domain;
	}

	/**
	 * Returns the plugin name.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return object $loader
	 */
	public function get_loader() {

		return $this->loader;
	}

	/** ------------------------------------------------------------------------ *
	 ** The functions that handle ajax requests.
	 ** ------------------------------------------------------------------------ */
	/**
	 * Adds a slideshow.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function add_slideshow() {

		if ( ! wp_verify_nonce($_REQUEST['nonce'], "cb_vegas_add_tab_nonce")) {
			exit(__("One more try and your browser will burst into flames ;-)", $this->plugin_domain));
		}

		// Get the new slideshow index.
		$new_slideshow_index = count(get_transient('cb_vegas_current_options_transient'));

		// Add slide.
		$html = $this->ajax->add_the_slideshow($new_slideshow_index);

		if (false !== $html) {

			$response = array(
				'success' => true,
				'message' => __("<h2>Ajax Action:</br>Slideshow added,</br>you need to save your changes or they will be lost.</h2>", $this->plugin_domain),
				'html'    => $html,
			);

			//$this->admin_notice_display();

			wp_send_json_success($response);
		} else {

			$response = array(
				'success' => false,
				'message' => __("<h2>Nope. Couldn't add a new slideshow :-(</br>Please save your work and / or refresh the page,</br>and then try again.</h2>", $this->plugin_domain),
			);

			wp_send_json_error($response);
		}
	}

	/**
	 * Adds a new slide to the slideshow.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function add_slide() {

		if ( ! wp_verify_nonce($_REQUEST['nonce'], "cb_vegas_add_slide_nonce")) {
			exit(__("One more try and your browser will burst into flames ;-)", $this->plugin_domain));
		}

		$slideshow_index = isset($_REQUEST['slideshow_index']) ? $_REQUEST['slideshow_index'] : false;
		$slide_index = isset($_REQUEST['slide_index']) ? $_REQUEST['slide_index'] : false;
		$html = $this->ajax->add_the_slide($slideshow_index, $slide_index);

		if (false !== $html) {

			$response = array(
				'success' => true,
				'message' => __("<h2>Ajax Action:</br>Slide added,</br>you need to save your changes or they will be lost.</h2>", $this->plugin_domain),
				'html'    => $html,
			);

			//$this->admin_notice_display();

			wp_send_json_success($response);
		} else {

			$response = array(
				'success' => false,
				'message' => __("<h2>Nope. Couldn't add a slide :-(</br>Please save your work and / or refresh the page,</br>and then try again.</h2>", $this->plugin_domain),
			);

			wp_send_json_error($response);
		}
	}

	/**
	 * Duplicates a slideshow.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function duplicate_slideshow() {

		if ( ! wp_verify_nonce($_REQUEST['nonce'], "cb_vegas_duplicate_slideshow_nonce")) {
			exit(__("One more try and your browser will burst into flames ;-)", $this->plugin_domain));
		}

		$slideshow_index = isset($_REQUEST['slideshow_index']) ? $_REQUEST['slideshow_index'] : false;
		$html = $this->ajax->duplicate_the_slideshow($slideshow_index);

		if (false !== $html) {

			$response = array(
				'success' => true,
				'message' => __("<h2>Ajax Action:</br>Slideshow duplicated,</br>you need to save your changes or they will be lost.</h2>", $this->plugin_domain),
				'html'    => $html,
			);

			//$this->admin_notice_display();

			wp_send_json_success($response);
		} else {

			$response = array(
				'success' => false,
				'message' => __("<h2>Nope. Couldn't duplicate the slideshow :-(</br>Please save your work and / or refresh the page,</br>and then try again.</h2>", $this->plugin_domain),
			);

			wp_send_json_error($response);
		}
	}

	/**
	 * Duplicates a slide.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function duplicate_slide() {

		if ( ! wp_verify_nonce($_REQUEST['nonce'], "cb_vegas_duplicate_slide_nonce")) {
			exit(__("One more try and your browser will burst into flames ;-)", $this->plugin_domain));
		}

		$slideshow_index = isset($_REQUEST['slideshow_index']) ? $_REQUEST['slideshow_index'] : false;
		$slide_index = isset($_REQUEST['slide_index']) ? $_REQUEST['slide_index'] : false;
		$html = $this->ajax->duplicate_the_slide($slideshow_index, $slide_index);

		if (false !== $html) {

			$response = array(
				'success' => true,
				'message' => __("<h2>Ajax Action:</br>Slide duplicated,</br>you need to save your changes or they will be lost.</h2>", $this->plugin_domain),
				'html'    => $html,
			);

			//$this->admin_notice_display();

			wp_send_json_success($response);
		} else {

			$response = array(
				'success' => false,
				'message' => __("<h2>Nope. Couldn't duplicate the slide :-(</br>Please save your work and / or refresh the page,</br>and then try again.</h2>", $this->plugin_domain),
			);

			wp_send_json_error($response);
		}
	}

	/**
	 * Sorts the slideshows.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function sort_slideshows() {

		if ( ! wp_verify_nonce($_REQUEST['nonce'], "cb_vegas_sort_tabs_nonce")) {
			exit(__("One more try and your browser will burst into flames ;-)", $this->plugin_domain));
		}

		$indices = $_REQUEST['indices'];
		$current_slideshow_index = $_REQUEST['slideshow_index'];

		if (true === $this->ajax->sort_the_slideshows($indices, $current_slideshow_index)) {

			$response = array(
				'success' => true,
				'message' => __("<h2>Ajax Action:</br>Slideshows rearranged,</br>you need to save your changes or they will be lost.</h2>", $this->plugin_domain),
			);

			//$this->admin_notice_display();

			wp_send_json_success($response);
		} else {

			$response = array(
				'success' => false,
				'message' => __("<h2>Nope. Couldn't sort the slideshows :-(</br>Please save your work and / or refresh the page,</br>and then try again.</h2>", $this->plugin_domain),
			);

			wp_send_json_error($response);
		}
	}

	/**
	 * Sorts the slides of the current slideshow.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function sort_slides() {

		if ( ! wp_verify_nonce($_REQUEST['nonce'], "cb_vegas_sort_slides_nonce")) {
			exit(__("One more try and your browser will burst into flames ;-)", $this->plugin_domain));
		}

		$slideshow_index = $_REQUEST['slideshow_index'];
		$indices = $_REQUEST['indices'];

		if (true === $this->ajax->sort_the_slides($slideshow_index, $indices)) {

			$response = array(
				'success' => true,
				'message' => __("<h2></h2>Ajax Action:</br>Slides rearranged,</br>you need to save your changes or they will be lost.</h2>", $this->plugin_domain),
			);

			//$this->admin_notice_display();

			wp_send_json_success($response);
		} else {

			$response = array(
				'success' => false,
				'message' => __("<h2>Nope. Couldn't sort the slides :-(</br>Please save your work and / or refresh the page,</br>and then try again.</h2>", $this->plugin_domain),
			);

			wp_send_json_error($response);
		}
	}

	/**
	 * Removes the current slideshow.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function remove_slideshow() {

		if ( ! wp_verify_nonce($_REQUEST['nonce'], 'cb_vegas_remove_slideshow_nonce')) {
			exit(__("One more try and your browser will burst into flames ;-)", $this->plugin_domain));
		}

		$deprecated_slideshow_index = isset($_REQUEST['deprecated_slideshow_index']) ? $_REQUEST['deprecated_slideshow_index'] : false;

		if (true === $this->ajax->remove_the_slideshow($deprecated_slideshow_index)) {

			$response = array(
				'success' => true,
				'message' => __("<h2>Ajax Action:</br>Slideshow removed.</br>You are being redirected.</h2>", $this->plugin_domain),
			);

			//$this->admin_notice_display();

			wp_send_json_success($response);
		} else {

			$response = array(
				'success' => false,
				'message' => __("<h2>Nope. Couldn't reorder anything :-(</br>Please save your work and / or refresh the page,</br>and then try again.</h2>", $this->plugin_domain),
			);

			wp_send_json_error($response);
		}
	}

	/**
	 * Removes a slide.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function remove_slide() {

		if ( ! wp_verify_nonce($_REQUEST['nonce'], "cb_vegas_remove_slide_nonce")) {
			exit(__("One more try and your browser will burst into flames ;-)", $this->plugin_domain));
		}

		// Get the slide index.
		$slideshow_index = isset($_REQUEST['slideshow_index']) ? $_REQUEST['slideshow_index'] : false;
		$slide_index = isset($_REQUEST['slide_index']) ? $_REQUEST['slide_index'] : false;

		if (true === $this->ajax->remove_the_slide($slideshow_index, $slide_index)) {

			$response = array(
				'success' => true,
				'message' => __("<h2>Ajax Action:</br>Slide removed,</br>you need to save your changes or they will be lost.</h2>", $this->plugin_domain),
			);

			//$this->admin_notice_display();

			wp_send_json_success($response);
		} else {

			$response = array(
				'success' => false,
				'message' => __("<h2>Nope. Couldn't remove the slide :-(</br>Please save your work and / or refresh the page,</br>and then try again.</h2>", $this->plugin_domain),
			);

			wp_send_json_error($response);
		}
	}

}
