<?php

/**
 * The class responsible for localizing the admin part of this plugin.
 *
 * @link              https://wordpress.org/plugins/cb-vegas/
 * @since             0.1.0
 * @package           CB_Vegas
 * @subpackage        CB_Vegas/admin/menu/includes
 * Author:            demispatti <demis@demispatti.ch>
 * Author URI:        http://demispatti.ch
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */
class CB_Vegas_Localisations {

	/**
	 * The name of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_name
	 */
	protected $plugin_name;
	
	/**
	 * The domain of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_domain
	 */
	protected $plugin_domain;

	/**
	 * Localizes texts concerning slide handling.
	 *
	 * @since  0.1.0
	 * @access private
	 * @param $plugin_name
	 * @param $plugin_domain
	 * @return void
	 */
	public function __construct($plugin_name, $plugin_domain) {

		$this->plugin_name = $plugin_name;
		$this->plugin_domain = $plugin_domain;
	}

	/**
	 * Kicks off localisation of the settings page and related elements.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function run() {

		$this->localize_main_script();
		$this->localize_ajax_script();
	}

	/**
	 * Localizes the settings page.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return array $texts
	 */
	private function localize_main_script() {

		wp_localize_script(
			$this->plugin_name . '-main-js',
			'cbVegas',
			array_merge(
				$this->get_checkbox_texts(),
				$this->get_main_script_texts()
			)
		);
	}

	/**
	 * Localizes an ajax related script.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return array $texts
	 */
	private function localize_ajax_script() {

		wp_localize_script(
			$this->plugin_name . '-ajax-js',
			'cbVegasAjax',
			array_merge(
				$this->get_ok_cancel_texts(),
				$this->get_ajax_script_texts()

			)
		);
	}

	/**
	 * Returns translated texts.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return array $texts
	 */
	private function get_main_script_texts() {

		$texts = array(
			'frameTitleText'                                => __('Select Media', $this->plugin_domain),
			'frameButtonText'                               => __('Set Media', $this->plugin_domain),
			'addMediaButtonText'                            => __("Add Media", $this->plugin_domain),
			'removeMediaButtonText'                         => __("Remove Media", $this->plugin_domain),
			'duplicateSlideshowSaveSettingsFirstButtonText' => __("Save your settings first, then duplicate the slideshow.", $this->plugin_domain)
		);

		return $texts;
	}

	/**
	 * Returns translated texts.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return array $texts
	 */
	private function get_ajax_script_texts() {

		$texts = array(
			'actionCancelledMessage'         => __("Nothing changed!", $this->plugin_domain),
			'addSlideshowConfirmation'       => __("Add Slideshow?", $this->plugin_domain),
			'dupliacteSlideshowConfirmation' => __("Duplicate Slideshow?", $this->plugin_domain),
			'removeSlideshowConfirmation'    => __("Remove Slideshow?", $this->plugin_domain),
			'addSlideshowButtonText'         => __("Add Slide", $this->plugin_domain),
			'removeSlideshowButtonText'      => __("Remove Slide", $this->plugin_domain),
			'addSlideConfirmation'           => __("Add Slide?", $this->plugin_domain),
			'duplicateSlideConfirmation'     => __("Duplicate Slide?", $this->plugin_domain),
			'removeSlideConfirmation'        => __("Remove Slide?", $this->plugin_domain),
			'addSlideButtonText'             => __("Add Slide", $this->plugin_domain),
			'removeSlideButtonText'          => __("Remove Slide", $this->plugin_domain)
		);

		return $texts;
	}

	/**
	 * Localizes texts for "Alertify".
	 *
	 * @since  0.1.0
	 * @access private
	 * @return array $texts
	 */
	private function get_ok_cancel_texts() {

		$texts = array(
			'okText'     => __('ok', $this->plugin_domain),
			'cancelText' => __('cancel', $this->plugin_domain),
		);

		return $texts;
	}

	/**
	 * Returns the texts for the switches (on/off switches).
	 *
	 * @since  0.1.0
	 * @access private
	 * @return array
	 */
	private function get_checkbox_texts() {

		$locale = get_locale();

		switch ($locale) {

			case($locale == 'de_DE');

				$texts = array(
					'locale'  => $locale,
					'onText'  => 'Ein',
					'offText' => 'Aus',
				);
				break;
			default:

				$texts = array(
					'locale'  => 'default',
					'onText'  => 'On',
					'offText' => 'Off',
				);
				break;
		}

		return $texts;
	}

}
