<?php

/**
 * The class responsible for the ajax functionality.
 *
 * @link              https://wordpress.org/plugins/cb-vegas/
 * @since             0.1.0
 * @package           CB_Vegas
 * @subpackage        CB_Vegas/admin/menu/includes/settings
 * Author:            demispatti <demis@demispatti.ch>
 * Author URI:        http://demispatti.ch
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */
class CB_Vegas_Ajax {

	/**
	 * The domain of the plugin.
	 *
	 * @since    0.1.0
	 * @access   public
	 * @var string $plugin_domain
	 */
	public $plugin_domain;

	/**
	 * The reference to the settings class.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var object $settings
	 */
	public $settings;

	/**
	 * The reference to the options class.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var object $vegas_options
	 */
	public $vegas_options;

	/**
	 * The reference to the class that generates all the settings.
	 *
	 * @since    0.1.0
	 * @access   protected
	 * @var object $settings_factory
	 */
	public $settings_factory;

	/**
	 * Kicks off the ajax functionality.
	 *
	 * @param $plugin_domain
	 * @param $vegas_options
	 * @param $settings
	 * @param $settings_factory
	 */
	public function __construct($plugin_domain, $vegas_options, $settings, $settings_factory) {

		$this->plugin_domain = $plugin_domain;
		$this->vegas_options = $vegas_options;
		$this->settings = $settings;
		$this->settings_factory = $settings_factory;
	}

	/**
	 * Adds a new slide.
	 *
	 * @param $slideshow_index
	 * @param $new_slide_index
	 * @return bool|string $html
	 */
	public function add_the_slide($slideshow_index, $new_slide_index) {

		$current_slideshows = get_transient('cb_vegas_current_options_transient');
		$html = '';
		// Registers the new settings fields with WordPress.
		$this->settings->add_settings_fields($slideshow_index, $new_slide_index);
		// Creates the settings fields for all the settings that make up a slide.
		foreach ($this->vegas_options->the_options as $setting => $args) {
			// Assigns the slideshow_index to the array of arguments so the function assigns the new slide to the current slideshow.
			$args['slideshow_index'] = $slideshow_index;
			// Creates the settings fields.
			$html .= $this->settings_factory->create_settings_fields($args, $new_slide_index);
		}
		// Retrieves the default options to populate the new slide with initial values.
		$the_new_slide = $this->settings_factory->create_slide_template($new_slide_index);
		// Assigns the new slide to it's array.
		$current_slideshows[ $slideshow_index ]['slides'][ $new_slide_index ] = $the_new_slide['slides'][ $new_slide_index ];

		if (true == set_transient('cb_vegas_current_options_transient', $current_slideshows)) {

			return $html;
		} else {

			return false;
		}
	}

	/**
	 * Adds a new slideshow.
	 *
	 * @param $new_slideshow_index
	 * @return bool|string $html
	 */
	public function add_the_slideshow($new_slideshow_index) {

		if (false !== $new_slideshow_index) {

			$current_slideshows = get_transient('cb_vegas_current_options_transient');
			// Retrieves the options for a complete slideshow with one slide.
			$new_slideshow = $this->settings_factory->create_slideshow($new_slideshow_index);
			// Adds the new slideshow to the array of slideshows.
			array_push($current_slideshows, $new_slideshow);
			// Extracts the name of the new slideshow.
			$slideshow_name = $new_slideshow['meta']['slideshow_name'];
			// Creates a new navigation tab for the slideshow and assigns its name to it.
			$html = $this->settings_factory->create_navigation_tab($new_slideshow_index, $slideshow_name);

			if (true === set_transient('cb_vegas_current_options_transient', $current_slideshows)) {

				return $html;
			} else {

				return false;
			}
		} else {

			return false;
		}
	}

	/**
	 * Duplicates the slide.
	 *
	 * @param $slideshow_index
	 * @param $slide_index
	 * @return bool|string $html
	 */
	public function duplicate_the_slide($slideshow_index, $slide_index) {

		$current_slideshows = get_transient('cb_vegas_current_options_transient');
		$copied_slide = $current_slideshows[ (int)$slideshow_index ]['slides'][ (int)$slide_index ];
		$new_slide_index = count($current_slideshows[ (int)$slideshow_index ]['slides']);
		$html = '';

		// Registers the new settings fields with WordPress.
		$this->settings->add_settings_fields($slideshow_index, $new_slide_index);

		// Creates the settings fields for all the settings that make up a slide.
		foreach ($this->vegas_options->the_options as $setting => $args) {
			// Assigns the slideshow_index to the array of arguments so the function assigns the new slide to the current slideshow.
			$args['slideshow_index'] = $slideshow_index;
			$value = $current_slideshows[ $slideshow_index ]['slides'][ $slide_index ][ $setting ];
			$args['value'] = $value;

			$html .= $this->settings_factory->create_settings_fields($args, $new_slide_index);
		}

		$current_slideshows[ $slideshow_index ]['slides'][ $new_slide_index ] = $copied_slide;

		if (true === set_transient('cb_vegas_current_options_transient', $current_slideshows)) {

			return $html;
		} else {

			return false;
		}
	}

	/**
	 * Duplicates the slideshow.
	 *
	 * @param $slideshow_index
	 * @return bool|string $html
	 */
	public function duplicate_the_slideshow($slideshow_index) {

		if (false !== $slideshow_index) {

			$uniqid = uniqid();

			$current_slideshows = get_transient('cb_vegas_current_options_transient');

			$copy = $current_slideshows[ $slideshow_index ];
			$copy['meta']['slideshow_uniqid'] = $uniqid;
			$copy['meta']['slideshow_name'] = $copy['meta']['slideshow_name'] . ' Copy';
			$copy['meta']['slideshow_is_global'] = false;
			$copy['meta']['slideshow_is_fallback'] = false;

			$slideshow_name = $copy['meta']['slideshow_name'];
			$new_slideshow_index = count($current_slideshows);
			$current_slideshows[ $new_slideshow_index ] = $copy;

			$html = $this->settings_factory->create_navigation_tab($new_slideshow_index, $slideshow_name);

			if (true === set_transient('cb_vegas_current_options_transient', $current_slideshows)) {

				return $html;
			} else {

				return false;
			}
		} else {

			return false;
		}
	}

	/**
	 * Rearranges the sldeshwows.
	 *
	 * @param $indices
	 * @param $current_slideshow_index
	 * @return bool
	 */
	public function sort_the_slideshows($indices, $current_slideshow_index) {

		$current_slideshows = get_transient('cb_vegas_current_options_transient');
		$list = [];
		$rearranged_slideshows = [];

		// Removes the substring so we have a list (an array) of old indices as "indices" containing the list of new indices as "values".
		$pattern = '/^cb-vegas-navigation-tab_/';
		foreach ($indices as $new => $old) {

			if (preg_match($pattern, $old)) {

				$old_index_number = preg_replace($pattern, '', $old);

				$list[ $new ] = $old_index_number;
			}
		}

		// Rearranges the slideshows.
		foreach ($list as $new_index => $old_index) {
			$rearranged_slideshows[ $new_index ] = $current_slideshows[ $old_index ];
		}

		// Updates the transient.
		delete_transient('cb_vegas_current_slideshow_index_transient');
		set_transient('cb_vegas_current_slideshow_index_transient', $current_slideshow_index, 1800);

		if (true === set_transient('cb_vegas_current_options_transient', $rearranged_slideshows)) {

			return true;
		} else {

			return false;
		}
	}

	/**
	 * Rearranges the slides.
	 *
	 * @param $slidshow_index
	 * @param $indices
	 * @return bool
	 */
	public function sort_the_slides($slidshow_index, $indices) {

		$current_slideshows = get_transient('cb_vegas_current_options_transient');
		$unsorted_slides = $current_slideshows[ $slidshow_index ]['slides'];
		$count = 0;
		$list = [];
		$sorted_slides = [];

		// Filters the indices and removes the pattern.
		$pattern = '/^cb-vegas-single-slide_/';
		foreach ($indices as $old_slideshow_index => $new_slideshow_index) {

			if ( ! preg_match($pattern, $new_slideshow_index)) {

				unset($old_slideshow_index);
			} else {

				$new_index = preg_replace($pattern, '', $new_slideshow_index);

				$list[ $count ] = $new_index;

				$count ++;
			}
		}

		// Rearranges the slides.
		foreach ($unsorted_slides as $index => $content) {

			$sorted_slides[ $index ] = $current_slideshows[ $slidshow_index ]['slides'][ $list[ $index ] ];
		}
		// Assigns the rearranged slides to its slideshow.
		$current_slideshows[ $slidshow_index ]['slides'] = $sorted_slides;

		if (true === set_transient('cb_vegas_current_options_transient', $current_slideshows)) {

			return true;
		} else {

			return false;
		}
	}

	/**
	 * Removes the slideshow.
	 *
	 * @param $deprecated_slideshow_index
	 * @return bool
	 */
	public function remove_the_slideshow($deprecated_slideshow_index) {

		$current_slideshows = get_transient('cb_vegas_current_options_transient');
		// Unsets the deprecated slideshow.
		unset($current_slideshows[ $deprecated_slideshow_index ]);
		// Sorts the slideshows.
		sort($current_slideshows);
		// Here we update the options because the user gets redirected to the slideshow with the lower index number.
		if (true === update_option('cb_vegas_options', $current_slideshows)) {

			return true;
		} else {

			return false;
		}
	}

	/**
	 * Removes the slide.
	 *
	 * @param $slideshow_index
	 * @param $slide_index
	 * @return bool
	 */
	public function remove_the_slide($slideshow_index, $slide_index) {

		$current_slideshows = get_transient('cb_vegas_current_options_transient');
		// Unsets the deprecated slide.
		unset($current_slideshows[ $slideshow_index ]['slides'][ $slide_index ]);
		// Sorts the slides.
		sort($current_slideshows[ $slideshow_index ]['slides']);

		if (true === set_transient('cb_vegas_current_options_transient', $current_slideshows)) {

			return true;
		} else {

			return false;
		}
	}

	/**
	 * Returns the domain of the plugin.
	 *
	 * @return string $plugin_domain
	 */
	public function get_plugin_domain() {

		return $this->plugin_domain;
	}

	/**
	 * Returns the reference to the class responsible for handling the settings.
	 *
	 * @return object $settings
	 */
	public function get_settings() {

		return $this->settings;
	}

}
