<?php

/**
 * The file that defines the core plugin class and bootstraps
 * the public and the admin facing parts of the plugin.
 *
 * @link       https://wordpress.org/plugins/cb-vegas/
 * @since      0.1.0
 * @package    CB_Vegas
 * @subpackage CB_Vegas/includes
 */

/**
 * The core plugin class.
 *
 * @since      0.1.0
 * @package    CB_Vegas
 * @subpackage CB_Vegas/includes
 * @author     demispatti
 */
class CB_Vegas {

	/**
	 * The name of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_name
	 */
	protected $plugin_name;

	/**
	 * The domain of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_domain
	 */
	protected $plugin_domain;

	/**
	 * The version of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_version
	 */
	protected $plugin_version;

	/**
	 * The reference to the class responsible
	 * for registering all hooks with WordPress.
	 *
	 * @since  0.1.0
	 * @access private
	 * @var    object $loader
	 */
	private $loader;

	/**
	 * Kicks of the plugin's main file.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function __construct() {

		$this->plugin_name = 'cb-vegas';
		$this->plugin_domain = 'cb-vegas';
		$this->plugin_version = '0.1.0';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Loads its dependencies and assigns the reference to the loader class.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function load_dependencies() {

		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cb-vegas-i18n.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-cb-vegas-admin.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-cb-vegas-public.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cb-vegas-loader.php';

		$this->loader = new CB_Vegas_Loader();
	}

	/**
	 * Sets the plugin domain.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function set_locale() {

		$plugin_i18n = new CB_Vegas_i18n();
		$plugin_i18n->set_plugin_domain($this->get_plugin_domain());

		$this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
	}

	/**
	 * Registers all admin hooks with WordPress.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function define_admin_hooks() {

		$Admin = new CB_Vegas_Admin($this->get_plugin_name(), $this->get_plugin_domain(), $this->get_plugin_version(), $this->get_loader());

		$this->loader->add_action('admin_enqueue_scripts', $Admin, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $Admin, 'enqueue_scripts');
		$this->loader->add_action('plugin_row_meta', $Admin, 'plugin_row_meta', 10, 2);
	}

	/**
	 * Registers all public hooks with WordPress.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return void
	 */
	private function define_public_hooks() {

		$Public = new CB_Vegas_Public($this->get_plugin_name(), $this->get_plugin_domain(), $this->get_loader());

		$this->loader->add_action('wp_enqueue_scripts', $Public, 'enqueue_styles', 40);
		$this->loader->add_action('wp_enqueue_scripts', $Public, 'enqueue_scripts', 400);
		$this->loader->add_action('wp_enqueue_scripts', $Public, 'localize_vegas', 1000);
	}

	/**
	 * Runs the loader.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function run() {

		$this->loader->run();
	}

	/**
	 * Returns the name of the plugin.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return string $plugin_name
	 */
	public function get_plugin_name() {

		return $this->plugin_name;
	}

	/**
	 * Returns the domain of the plugin.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return string $plugin_domain
	 */
	public function get_plugin_domain() {

		return $this->plugin_domain;
	}

	/**
	 * Returns the version of the plugin.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return string $plugin_version
	 */
	public function get_plugin_version() {

		return $this->plugin_version;
	}

	/**
	 * Returns the reference to the loader object.
	 *
	 * @since  0.1.0
	 * @access public
	 * @return object $loader
	 */
	public function get_loader() {

		return $this->loader;
	}

}
