<?php

/**
 * The class responsible for localizing the "Vegas Background Slideshow" configuration file.
 *
 * @link              https://wordpress.org/plugins/cb-vegas/
 * @since             0.1.0
 * @package           CB_Vegas
 * @subpackage        CB_Vegas/includes
 * Author:            Demis Patti <demis@demispatti.ch>
 * Author URI:        http://demispatti.ch
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */
class CB_Vegas_Localisation {

	/**
	 * The name of the plugin.
	 *
	 * @since  0.1.0
	 * @access protected
	 * @var    string $plugin_name
	 */
	protected $plugin_name;

	/**
	 * Kicks off the localisation of the "Vegas Background Slideshow" configuration file.
	 *
	 * @since  0.1.0
	 * @access public
	 * @param  $plugin_name
	 * @return void
	 */
	public function __construct($plugin_name) {

		$this->plugin_name = $plugin_name;
	}

	/**
	 * Delivers the configuration data and the overlay url to the "Vegas Background Slideshow" library.
	 *
	 * @hooked_action
	 *
	 * @since  0.1.0
	 * @access public
	 * @return void
	 */
	public function localize_vegas($current_slideshow) {

		wp_localize_script($this->plugin_name . '-localisation-js',
			'Vegas',
			array_merge(
				$this->get_configuration($current_slideshow),
				$this->get_overlays_url()
			)
		);
	}

	/**
	 * Retrieves the configuration data.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return array $configuration
	 */
	private function get_configuration($current_slideshow) {

		if (false === get_option('cb_vegas_options') || true !== is_array(get_option('cb_vegas_options'))) {

			return false;
		} else {

			$slides = [];
			$settings = [];
			$options = get_option('cb_vegas_options');

			// Arranges the configuration to serve Vegas in an optimal way.
			foreach ($options[ $current_slideshow ]['slides'] as $index => $setting) {

				foreach ($setting as $key => $value) {

					if ($key === 'src') {

						// Array containing the list of slides.
						$slides[ $index ] = '{src: ' . site_url() . $value . '}';
						// Array containing the settings per slide stored in the list above.
						$settings[ $index ][ $key ] = $value;
						$slides[ $index ] = $value;
					} else {

						$settings[ $index ][ $key ] = $value;
					}

					if($key === 'delay'){
						$alternateKeyForDelay = 'slideDelay';
						$settings[ $index ][ $alternateKeyForDelay ] = $value;
						unset($settings[ $index ][ $key ]);
					}
				}
			}

			// Add the meta data.
			$meta = $options[ $current_slideshow ]['meta'];
			$meta['overlay_path'] = $this->get_overlays_url();

			// Assign the previously created arrays to the configuration variable.
			$configuration = ['meta' => $meta, 'slides' => $slides, 'settings' => $settings];

			// Removes slides w/o assigned media.
			foreach ($configuration['settings'] as $index => $slide) {

				if (false === $slide['src'] || '' === $slide['src']) {

					unset($configuration['settings'][ $index ]);
					unset($configuration['slides'][ $index ]);
				}
			}

			ksort($configuration['settings']);

			return $configuration;
		}
	}

	/**
	 * Retrieves the path to the overlay image.
	 *
	 * @since  0.1.0
	 * @access private
	 * @return array $path
	 */
	private function get_overlays_url() {

		$overlay_path = site_url() . '/wp-content/plugins/cb-vegas/vendor/vegas-slideshow/overlays/';
		$path = array('overlay_path' => $overlay_path);

		return $path;
	}

}
